/********************************************************************************************************
Macro Name:   normDesignPP.sas
Author:       Matthew Psioda
Affiliation:  Department of Biostatistics, University of North Carolina at Chapel Hill
    
**** Program Summary:
     This program calculates the power/type I error rates based on simulated normal studies and using the power prior

**** Needed Macro Variables;
    (1) MCMCOptions     = <text> options for proc MCMC - ex: %str(nbi=500 nmc=20000 nthin=1)
    (2) gamma           = <double> lower bound for posterior probability decision rule
    (3) delta           = <double> non-inferiority margin
    (4) dsIn            = <libname.dataset> dataset containing simulation studies for analysis
    (5) dsInPP          = <libname.dataset> dataset containing historical studies (sufficient statistics)
    (6) dsOut           = <libname.dataset> dataset containing empirical power/type one error rates
*********************************************************************************************************/

%macro NormDesignPP(MCMCOptions=,gamma=0.95,delta=0.020,dsIn=simStudy,dsInPP=powerPrior,dsOut=Power);

data _null_;
 set  &dsInPP.;
 call symput('hc_k',strip(put(_n_,best.)));
run;

options cmplib=work.funcs;
proc fcmp outlib=work.funcs.uds;

   subroutine Gibbs(mu_t,mu_c,vr,Yt_mean,S2_t,Nt,Yc_mean,S2_c,Nc,n0a0,muc0bar,S20a0);
   outargs mu_t,mu_c,vr;

    mu_t = rand('Normal',Yt_mean,sqrt(vr/Nt));
   
    varNew = 1/(Nc/vr + n0a0/vr);
    muNew  = (Nc/vr * Yc_mean + n0a0/vr*muc0bar)*varNew;
	   mu_c = rand('Normal',muNew,sqrt(varNew));

    shape = 0.5*(Nt+Nc+n0a0);
    rate  = 0.5*( (mu_c-muc0bar)**2 + S20a0 + (S2_t - 2*Nt*Yt_mean*mu_t + Nt*mu_t**2) + (S2_c - 2*Nc*Yc_mean*mu_c + Nc*mu_c**2) );
       vr   = 1 / ( rand('Gamma',shape)/rate );
   endsub;
run;

options cmplib=work.funcs;

ods output PostSummaries=PostSummaries;
proc mcmc data = simStudy(drop=mu:) monitor=(mu_t mu_c vr diff indicator) statistics=summary &mcmcOptions.;
 by StudyRep;

 array hc_n[&hc_k.]  / nosymbols;
 array hc_mu[&hc_k.] / nosymbols;
 array hc_vr[&hc_k.] / nosymbols;
 array hc_a0[&hc_k.] / nosymbols;

  begincnst;
   rc = read_array("&dsInPP.", hc_n, "n");
   rc = read_array("&dsInPP.", hc_mu, "mu");
   rc = read_array("&dsInPP.", hc_vr, "vr");
   rc = read_array("&dsInPP.", hc_a0, "a0");

   n0a0    = 0;
   muc0bar = 0;  
   S20a0   = 0;

   do k = 1 to &hc_k.;
     n0a0    = n0a0 + hc_a0[k]*hc_n[k];
     muc0bar = muc0bar + hc_a0[k]*hc_n[k]*hc_mu[k];
   end;
   muc0bar = muc0bar / n0a0;

   do k = 1 to &hc_k.;
     S20a0   = S20a0 + hc_a0[k]*hc_n[k]*(hc_mu[k] - muc0bar)**2  
                     + hc_a0[k]*(hc_n[k]-1)*hc_vr[k];
   end;
   
    S2_t = Yt_SSE + Nt*Yt_Mean**2;
    S2_c = Yc_SSE + Nc*Yc_Mean**2;    
  endcnst;
  
  beginnodata;
   diff      = mu_t-mu_c;
   indicator = (diff < &delta);
  endnodata;
  
  parms mu_c 0.0 mu_t 0.0 vr 1 / uds;

  prior mu_t mu_c vr ~ general(0);

  uds Gibbs(mu_t,mu_c,vr,Yt_mean,S2_t,Nt,Yc_mean,S2_c,Nc,n0a0,muc0bar,S20a0);



  model general(0);
run;

data &dsOut.;
 set PostSummaries;
 where Parameter = 'indicator';
 Rej = ( Mean > &gamma.); 
run;
proc univariate data = &dsOut. noprint;
 var Rej;
 output out = &dsOut. mean=RejRate;
run;

data _temp_(drop=Ntot);
 set &dsIN.(obs=1);
 keep mu: n:;
run;

data &dsOut.;
 merge _temp_ &dsOut.;
 label RejRate = 'Empirical Null Hypothesis Rejection Rate';
run;

proc datasets library=work noprint; delete _temp_; run; quit;
%mend;
